@@ -2,35 +2,16 @@
 
 from dataclasses import dataclass
 from datetime import datetime
-from typing import List, Optional, Union
+from typing import List, Literal, Optional, Union
 
 import apibara.application.indexer_service_pb2 as indexer_service_pb2
 from apibara.starknet import get_selector_from_name
 
 
-@dataclass
-class Topic:
-    choices: List[bytes]
-
-    @staticmethod
-    def from_proto(p: indexer_service_pb2.Topic):
-        return Topic([c.value for c in p.choices])
-
-    def to_proto(self) -> indexer_service_pb2.Topic:
-        return indexer_service_pb2.Topic(
-            choices=[indexer_service_pb2.TopicValue(value=c) for c in self.choices]
-        )
-
-    def __str__(self) -> str:
-        choices = [f"0x{c.hex()}" for c in self.choices]
-        return f'Topic({"|".join(choices)})'
-
-
 @dataclass
 class EventFilter:
-    name: Optional[str]
+    signature: str
     address: Optional[bytes]
-    topics: List[Topic]
 
     @classmethod
     def from_event_name(
@@ -39,36 +20,43 @@ class EventFilter:
         """Create an EventFilter from the event name."""
         if isinstance(address, str):
             address = bytes.fromhex(address.replace("0x", ""))
-        topic_hex = hex(get_selector_from_name(name))[2:]
-        # fromhex requires an even number of digits
-        if len(topic_hex) % 2 == 1:
-            topic_hex = "0" + topic_hex
-        topic_value = bytes.fromhex(topic_hex)
-        topics = [Topic([topic_value])]
-
-        return cls(name, address, topics)
+        return cls(name, address)
 
     @staticmethod
     def from_proto(p: indexer_service_pb2.EventFilter):
         address = None
         if len(p.address) > 0:
             address = p.address
-
-        topics = [Topic.from_proto(t) for t in p.topics]
-        return EventFilter(name=None, address=address, topics=topics)
+        return EventFilter(signature=p.signature, address=address)
 
     def to_proto(self) -> indexer_service_pb2.EventFilter:
         return indexer_service_pb2.EventFilter(
-            address=self.address, topics=[t.to_proto() for t in self.topics]
+            signature=self.signature,
+            address=self.address,
         )
 
     def __str__(self) -> str:
         address = ""
         if self.address is not None:
-            address = f"address=0x{self.address.hex()}, "
-        return (
-            f'EventFilter({address}topics=[{", ".join(str(t) for t in self.topics)}])'
-        )
+            address = f", address=0x{self.address.hex()}"
+        return f"EventFilter(signature={self.signature}{address})"
+
+
+@dataclass
+class Network:
+    name: str
+    type: Literal["evm", "starknet"]
+
+    @staticmethod
+    def from_proto(p: indexer_service_pb2.Network):
+        if p.HasField("starknet"):
+            return Network(p.starknet.name, "starknet")
+        if p.HasField("ethereum"):
+            return Network(p.ethereum.name, "evm")
+        raise ValueError("invalid network proto object")
+
+    def __str__(self) -> str:
+        return f"Network(name={self.name}, type={self.type})"
 
 
 @dataclass
@@ -77,24 +65,39 @@ class Indexer:
     index_from_block: int
     indexed_to_block: int
     filters: List[EventFilter]
+    network: Network
 
     @staticmethod
     def from_proto(p: indexer_service_pb2.Indexer):
         filters = [EventFilter.from_proto(f) for f in p.filters]
-        return Indexer(p.id, p.index_from_block, p.indexed_to_block, filters)
+        network = Network.from_proto(p.network)
+        return Indexer(p.id, p.index_from_block, p.indexed_to_block, filters, network)
 
     def __str__(self) -> str:
-        return f'Indexer(id={self.id}, blocks=[{self.index_from_block}, {self.indexed_to_block}], filters=[{", ".join(str(f) for f in self.filters)}])'
+        return f'Indexer(id={self.id}, network={self.network}, blocks=[{self.index_from_block}, {self.indexed_to_block}], filters=[{", ".join(str(f) for f in self.filters)}])'
+
+
+@dataclass
+class Version:
+    major: int
+    minor: int
+    patch: int
+
+    @staticmethod
+    def from_proto(p: indexer_service_pb2.Version):
+        return Version(major=p.major, minor=p.minor, patch=p.patch)
 
 
 @dataclass
 class IndexerConnected:
     indexer: Indexer
+    version: Version
 
     @staticmethod
     def from_proto(p: indexer_service_pb2.IndexerConnected):
-        app = Indexer.from_proto(p.indexer)
-        return IndexerConnected(app)
+        indexer = Indexer.from_proto(p.indexer)
+        version = Version.from_proto(p.version)
+        return IndexerConnected(indexer=indexer, version=version)
 
 
 @dataclass
@@ -115,23 +118,65 @@ class BlockHeader:
 
 @dataclass
 class Event:
+    @staticmethod
+    def from_proto(p: indexer_service_pb2.Event):
+        if p.HasField("starknet"):
+            return StarkNetEvent.from_proto(p.starknet)
+        if p.HasField("ethereum"):
+            return EthereumEvent.from_proto(p.ethereum)
+
+
+@dataclass
+class StarkNetEvent(Event):
     name: Optional[str]
     address: bytes
-    block_index: int
+    log_index: int
     topics: List[bytes]
     data: List[bytes]
 
     @staticmethod
-    def from_proto(p: indexer_service_pb2.Event):
+    def from_proto(p: indexer_service_pb2.StarkNetEvent):
         topics = [bytes(t.value) for t in p.topics]
         data = [bytes(d.value) for d in p.data]
-        return Event(None, bytes(p.address), p.block_index, topics, data)
+        return StarkNetEvent(
+            name=None,
+            address=bytes(p.address),
+            log_index=p.log_index,
+            topics=topics,
+            data=data,
+        )
 
     def __str__(self) -> str:
         name = ""
         if self.name is not None:
             name = f"name={self.name}, "
-        return f"Event({name}address=0x{self.address.hex()}, block_index={self.block_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"
+        return f"StarkNetEvent({name}address=0x{self.address.hex()}, log_index={self.log_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"
+
+
+@dataclass
+class EthereumEvent(Event):
+    name: Optional[str]
+    address: bytes
+    log_index: int
+    topics: List[bytes]
+    data: bytes
+
+    @staticmethod
+    def from_proto(p: indexer_service_pb2.EthereumEvent):
+        topics = [bytes(t.value) for t in p.topics]
+        return EthereumEvent(
+            name=None,
+            address=bytes(p.address),
+            log_index=p.log_index,
+            topics=topics,
+            data=p.data,
+        )
+
+    def __str__(self) -> str:
+        name = ""
+        if self.name is not None:
+            name = f"name={self.name}, "
+        return f"EthereumEvent({name}address=0x{self.address.hex()}, log_index={self.log_index}, ...{len(self.topics)} topics, ...{len(self.data)} data)"
 
 
 @dataclass
@@ -162,14 +207,14 @@ class Reorg:
 
 @dataclass
 class NewEvents:
-    block_hash: bytes
-    block_number: int
+    block: BlockHeader
     events: List[Event]
 
     @staticmethod
     def from_proto(p: indexer_service_pb2.NewEvents):
+        block = BlockHeader.from_proto(p.block)
         events = [Event.from_proto(ev) for ev in p.events]
-        return NewEvents(bytes(p.block_hash), p.block_number, events)
+        return NewEvents(block, events)
 
     def __str__(self) -> str:
-        return f"NewEvents(block_hash=0x{self.block_hash.hex()}, block_number={self.block_number}, ...{len(self.events)} events)"
+        return f"NewEvents(block={self.block}, ...{len(self.events)} events)"
